@@ -1,26 +1,27 @@
 ;;; erc-match.el --- Highlight messages matching certain regexps
 
-;; Copyright (C) 2002,2003,2004 Free Software Foundation, Inc.
-;; Copyright (C) 2004 Jeremy Bertram Maitin-Shepard
+;; Copyright (C) 2002, 2003, 2004, 2005 Free Software Foundation, Inc.
 
 ;; Author: Andreas Fuchs <asf@void.at>
 ;; Keywords: comm, faces
 ;; URL: http://www.emacswiki.org/cgi-bin/wiki.pl?ErcMatch
 
-;; This file is free software; you can redistribute it and/or modify
+;; This file is part of GNU Emacs.
+
+;; GNU Emacs is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
 ;; the Free Software Foundation; either version 2, or (at your option)
 ;; any later version.
 
-;; This file is distributed in the hope that it will be useful,
+;; GNU Emacs is distributed in the hope that it will be useful,
 ;; but WITHOUT ANY WARRANTY; without even the implied warranty of
 ;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ;; GNU General Public License for more details.
 
 ;; You should have received a copy of the GNU General Public License
-;; along with GNU Emacs; see the file COPYING.  If not, write to
-;; the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-;; Boston, MA 02111-1307, USA.
+;; along with GNU Emacs; see the file COPYING.  If not, write to the
+;; Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+;; Boston, MA 02110-1301, USA.
 
 ;;; Commentary:
 
@@ -35,12 +36,10 @@
 ;;; Code:
 
 (require 'erc)
+(eval-when-compile (require 'cl))
 
 ;; Customisation:
 
-(defconst erc-match-version "$Revision: 1.38.2.3 $"
-  "ERC match mode revision.")
-
 (defgroup erc-match nil
   "Keyword and Friend/Foe/... recognition.
 Group containing all things concerning pattern matching in ERC
@@ -55,7 +54,7 @@ they are hidden or highlighted.  This is controlled via the variables
 `erc-current-nick-highlight-type'.  For all these highlighting types,
 you can decide whether the entire message or only the sending nick is
 highlighted."
-  ((add-hook 'erc-insert-modify-hook 'erc-match-message))
+  ((add-hook 'erc-insert-modify-hook 'erc-match-message 'append))
   ((remove-hook 'erc-insert-modify-hook 'erc-match-message)))
 
 ;; Remaining customizations
@@ -85,7 +84,7 @@ Useful to mark nicks from dangerous hosts."
   :group 'erc-match
   :type '(repeat regexp))
 
-(defcustom erc-current-nick-highlight-type nil
+(defcustom erc-current-nick-highlight-type 'keyword
   "*Determines how to highlight text in which your current nickname appears
 \(does not apply to text sent by you\).
 
@@ -213,11 +212,25 @@ be formatted. The various format specs are:
   :group 'erc-match
   :type 'string)
 
-(defcustom erc-text-matched-hook '(erc-log-matches erc-hide-fools)
+(defcustom erc-beep-match-types '(current-nick)
+  "Types of matches to beep for when a match occurs.
+The function `erc-beep-on-match' needs to be added to `erc-text-matched-hook'
+for beeping to work."
+  :group 'erc-match
+  :type '(choice (repeat :tag "Beep on match" (choice
+					       (const current-nick)
+					       (const keyword)
+					       (const pal)
+					       (const dangerous-host)
+					       (const fool)))
+		 (const :tag "Don't beep" nil)))
+
+(defcustom erc-text-matched-hook '(erc-log-matches)
   "Hook run when text matches a given match-type.
 Functions in this hook are passed as arguments:
 \(match-type nick!user@host message) where MATCH-TYPE is a symbol of:
-keyword, pal, dangerous-host, fool"
+current-nick, keyword, pal, dangerous-host, fool"
+  :options '(erc-log-matches erc-hide-fools erc-beep-on-match)
   :group 'erc-match
   :type 'hook)
 
@@ -326,7 +339,8 @@ car is the string."
 (defun erc-add-fool ()
   "Add fool interactively to `erc-fools'."
   (interactive)
-  (erc-add-entry-to-list 'erc-fools "Add fool: " (erc-get-server-nickname-alist)))
+  (erc-add-entry-to-list 'erc-fools "Add fool: "
+			 (erc-get-server-nickname-alist)))
 
 ;;;###autoload
 (defun erc-delete-fool ()
@@ -363,7 +377,7 @@ car is the string."
 NICKUSERHOST will be ignored."
   (with-syntax-table erc-match-syntax-table
     (and msg
-         (string-match (concat "\\b"
+	 (string-match (concat "\\b"
 			       (regexp-quote (erc-current-nick))
 			       "\\b")
 		       msg))))
@@ -414,24 +428,6 @@ In any of the following situations, MSG is directed at an entry FOOL:
     (or (erc-list-match fools-beg msg)
 	(erc-list-match fools-end msg))))
 
-(defun erc-get-parsed-vector (point)
-  "Return the whole parsed vector on POINT."
-  (get-text-property point 'erc-parsed))
-
-(defun erc-get-parsed-vector-nick (vect)
-  "Return nickname in the parsed vector VECT."
-  (let* ((untreated-nick (and vect (erc-response.sender vect)))
-	 (maybe-nick (when untreated-nick
-		       (car (split-string untreated-nick "!")))))
-    (when (and (not (null maybe-nick))
-	       (erc-is-valid-nick-p maybe-nick))
-      untreated-nick)))
-
-(defun erc-get-parsed-vector-type (vect)
-  "Return message type in the parsed vector VECT."
-  (and vect
-       (erc-response.command vect)))
-
 (defun erc-match-message ()
   "Mark certain keywords in a region.
 Use this defun with `erc-insert-modify-hook'."
@@ -469,35 +465,35 @@ Use this defun with `erc-insert-modify-hook'."
 		(match-face (intern (concat match-prefix "-face"))))
 	   (when (funcall match-pred nickuserhost message)
 	     (cond
-              ;; Highlight the nick of the message
+	      ;; Highlight the nick of the message
 	      ((and (eq match-htype 'nick)
-                    nick-end)
+		    nick-end)
 	       (erc-put-text-property
 		nick-beg nick-end
 		'face match-face (current-buffer)))
-              ;; Highlight the nick of the message, or the current
-              ;; nick if there's no nick in the message (e.g. /NAMES
-              ;; output)
-              ((and (string= match-type "current-nick")
-                    (eq match-htype 'nick-or-keyword))
-               (if nick-end 
-                   (erc-put-text-property
-                    nick-beg nick-end
-                    'face match-face (current-buffer))
-                 (goto-char (+ 2 (or nick-end
-                                     (point-min))))
-                 (while (re-search-forward match-regex nil t)
-                   (erc-put-text-property (match-beginning 0) (match-end 0)
-                                          'face match-face))))
-              ;; Highlight the whole message
+	      ;; Highlight the nick of the message, or the current
+	      ;; nick if there's no nick in the message (e.g. /NAMES
+	      ;; output)
+	      ((and (string= match-type "current-nick")
+		    (eq match-htype 'nick-or-keyword))
+	       (if nick-end
+		   (erc-put-text-property
+		    nick-beg nick-end
+		    'face match-face (current-buffer))
+		 (goto-char (+ 2 (or nick-end
+				     (point-min))))
+		 (while (re-search-forward match-regex nil t)
+		   (erc-put-text-property (match-beginning 0) (match-end 0)
+					  'face match-face))))
+	      ;; Highlight the whole message
 	      ((eq match-htype 'all)
 	       (erc-put-text-property
 		(point-min) (point-max)
 		'face match-face (current-buffer)))
-              ;; Highlight all occurrences of the word to be
-              ;; highlighted.
+	      ;; Highlight all occurrences of the word to be
+	      ;; highlighted.
 	      ((and (string= match-type "keyword")
-                    (eq match-htype 'keyword))
+		    (eq match-htype 'keyword))
 	       (mapc (lambda (elt)
 		       (let ((regex elt)
 			     (face match-face))
@@ -511,7 +507,7 @@ Use this defun with `erc-insert-modify-hook'."
 			    (match-beginning 0) (match-end 0)
 			    'face face))))
 		     match-regex))
-              ;; Highlight all occurrences of our nick.
+	      ;; Highlight all occurrences of our nick.
 	      ((and (string= match-type "current-nick")
 		    (eq match-htype 'keyword))
 	       (goto-char (+ 2 (or nick-end
@@ -519,7 +515,7 @@ Use this defun with `erc-insert-modify-hook'."
 	       (while (re-search-forward match-regex nil t)
 		 (erc-put-text-property (match-beginning 0) (match-end 0)
 					'face match-face)))
-              ;; Else twiddle your thumbs.
+	      ;; Else twiddle your thumbs.
 	      (t nil))
 	     (run-hook-with-args
 	      'erc-text-matched-hook
@@ -531,9 +527,6 @@ Use this defun with `erc-insert-modify-hook'."
 	   (append to-match-nick-dep to-match-nick-indep)
 	 to-match-nick-indep)))))
 
-;; used in erc.el for session buffers only -- silence the compiler, here
-(defvar away)
-
 (defun erc-log-matches (match-type nickuserhost message)
   "Log matches in a separate buffer, determined by MATCH-TYPE.
 The behaviour of this function is controlled by the variables
@@ -547,13 +540,15 @@ deactivate/activate match logging in the latter. See
     (when (and
 	   (or (eq erc-log-matches-flag t)
 	       (and (eq erc-log-matches-flag 'away)
-		    away))
+		    erc-away))
 	   match-buffer-name)
       (let ((line (format-spec erc-log-match-format
 		   (format-spec-make
 		    ?n nick
-		    ?t (format-time-string (or erc-timestamp-format
-					       "[%Y-%m-%d %H:%M] "))
+		    ?t (format-time-string
+			(or (and (boundp 'erc-timestamp-format)
+				 erc-timestamp-format)
+			    "[%Y-%m-%d %H:%M] "))
 		    ?c (or (erc-default-target) "")
 		    ?m message
 		    ?u nickuserhost))))
@@ -566,19 +561,19 @@ deactivate/activate match logging in the latter. See
 (defun erc-log-matches-make-buffer (name)
   "Create or get a log-matches buffer named NAME and return it."
   (let* ((buffer-already (get-buffer name))
-	 (buffer         (or buffer-already
-			     (get-buffer-create name))))
+	 (buffer (or buffer-already
+		     (get-buffer-create name))))
     (with-current-buffer buffer
       (unless buffer-already
 	(insert " == Type \"q\" to dismiss messages ==\n")
-	(view-mode-enter nil (lambda (buffer)
-			       (when (y-or-n-p "Discard messages?")
-				 (kill-buffer buffer)))))
+	(erc-view-mode-enter nil (lambda (buffer)
+				   (when (y-or-n-p "Discard messages?")
+				     (kill-buffer buffer)))))
       buffer)))
 
 (defun erc-log-matches-come-back (proc parsed)
   "Display a notice that messages were logged while away."
-  (when (and away
+  (when (and erc-away
 	     (eq erc-log-matches-flag 'away))
     (mapc
      (lambda (match-type)
@@ -589,7 +584,7 @@ deactivate/activate match logging in the latter. See
 				  (with-current-buffer buffer
 				    (get-text-property (1- (point-max))
 						       'timestamp))))
-		  (away-time     (erc-emacs-time-to-erc-time away)))
+		  (away-time     (erc-emacs-time-to-erc-time erc-away)))
 	     (when (and away-time last-msg-time
 			(erc-time-gt last-msg-time away-time))
 	       (erc-display-message
@@ -627,6 +622,12 @@ This function should be called from `erc-text-matched-hook'."
 			    '(invisible intangible)
 			    (current-buffer))))
 
+(defun erc-beep-on-match (match-type nickuserhost message)
+  "Beep when text matches.
+This function is meant to be called from `erc-text-matched-hook'."
+  (when (member match-type erc-beep-match-types)
+    (beep)))
+
 (provide 'erc-match)
 
 ;;; erc-match.el ends here
@@ -634,5 +635,6 @@ This function should be called from `erc-text-matched-hook'."
 ;; Local Variables:
 ;; indent-tabs-mode: t
 ;; tab-width: 8
-;; standard-indent: 4
 ;; End:
+
+;; arch-tag: 1f1f595e-abcc-4b0b-83db-598a1d3f0f82
